@@ -18,7 +18,7 @@ function utf8_substr_replace($str, $repl, $start , $length=null)
 	preg_match_all('/./us', $str, $ar);
 	preg_match_all('/./us', $repl, $rar);
 
-	if($length === null)
+	if(is_null($length))
 		$length = utf8_strlen($str);
 
 	array_splice($ar[0], $start, $length, $rar[0]);
