@@ -3,15 +3,14 @@
 // Language definitions used in admin_censoring.php
 $lang_admin_censoring = array(
 
-'Must enter both message'	=>	'You must enter both a word to censor and text to replace it with.',
-'Must search both message'	=>	'You must enter both text to search for and text to replace with.',
+'Must enter word message'	=>	'You must enter a word to censor.',
 'Word updated redirect'		=>	'Censor word updated. Redirecting …',
 'Word added redirect'		=>	'Censor word added. Redirecting …',
 'Word removed redirect'		=>	'Censor word removed. Redirecting …',
 'Censoring head'			=>	'Censoring',
 'Add word subhead'			=>	'Add word',
 'Add word info'				=>	'Enter a word that you want to censor and the replacement text for this word. Wildcards are accepted (i.e. *some* would match somewhere and lonesome). Censor words also affect usernames. New users will not be able to register with usernames containing any censored words. The search is case insensitive.',
-'Censoring enabled'		=>	'<strong>Censoring is enabled in %s.</strong>',
+'Censoring enabled'			=>	'<strong>Censoring is enabled in %s.</strong>',
 'Censoring disabled'		=>	'<strong>Censoring is disabled in %s.</strong>',
 'Censored word label'		=>	'Censored word',
 'Replacement label'			=>	'Replacement word(s)',
